# coding: utf-8

from __future__ import print_function, unicode_literals

import os
import shutil

import tst_wrapper


def create_exercise_file(name, path, exercise, student):
    # Add an extension to the file
    python_file = name + '.py'
    os.chdir(path)

    # Get the full name of the exercise and return it
    path_python_file = '{}{}'.format(path, python_file)

    # Writes the header in the file
    if is_zero_file(path_python_file):
        with open(python_file, 'a') as f:
            f.write(header(exercise, student))


def exercise_path(exercise):
    tst_exercises = tst_wrapper.EXERCISES_DIR
    return os.path.join(os.path.expanduser('~/'),
                        tst_exercises,
                        format_unit(exercise),
                        exercise['name'])


def format_unit(exercise):
    # Get raw unit from exercise
    unit_raw = exercise['unit'].encode('utf-8')
    unit = ''

    # Get unit number from unit_raw
    for char in unit_raw:
        if char.isdigit():
            unit += char
    # Format in two chars
    unit = 'unidade' + unit.zfill(2)

    # Return unit from exercise
    return unit


def header(exercise, student):
    result = '''# coding: utf-8

##{:44}##
# {:44} #
# {:44} #
# {:44} #
# {:44} #
# {:44} #
##{:44}##


'''.format('#' * 44,
           'Disciplina: Programação 01 - 2018.1',
           'Nome: ' + student['name'],
           'E-mail: ' + student['email'],
           'Atividade: ' + exercise['label'],
           'Unidade: ' + format_unit(exercise)[-2::],
           '#' * 44
           )

    # Encode is needed because of accents on Programação
    return result.encode('utf-8')


def is_zero_file(fpath):
    return not (os.path.isfile(fpath) and os.path.getsize(fpath) > 0)


def is_logged_in():
    r = tst_wrapper.request_to_tst()

    logged_in = (r.status_code != 400)

    if not logged_in:
        print('Please log on tst (Run tst login).')
        raise RuntimeError('Not logged in tst.')


def rename_directory(base, destination):
    # Rename directory
    if not os.path.isdir(destination):
        shutil.move(base, destination)
    else:
        # FIXME: Erasing directory to update it. Currently a workaround.
        print('''
You already have a directory for this exercise with the following path:
{}

This directory will be erased and updated with the latest commit.'''.format(destination))

        while True:
            decision = raw_input('Are you sure you want to proceed with this operation? (y/n) ').strip().lower()
            if decision == 'y':
                shutil.rmtree(destination)
                shutil.move(base, destination)
                break
            elif decision == 'n':
                shutil.rmtree(base)
                break
            else:
                print('Please, just type "y" or "n"')
        print()

    return destination



# coding: utf-8

from __future__ import print_function, unicode_literals

import pyperclip
import time

import tst_wrapper
import easy_helper


def clipboard_try():
    # Get what is in clipboard and remove spaces
    clipboard = pyperclip.paste().replace(' ', '')

    # Check if clipboard is a valid code
    if len(clipboard) == 4:  # Checkout codes always have 4 letters/numbers
        # Try checkout with code
        try:
            tst_wrapper.main(clipboard)
        except IndexError:
            print('"{}" is not a valid code.'.format(clipboard))
    else:
        print('Trying to checkout: "{}..."'.format(clipboard[:10]).replace('\n', ' '))
        print('Code does not meet base criteria. Skipping...')


def watcher():
    clipboard = pyperclip.paste()

    try:
        started = False
        while True:
            # Define old clipboard value
            old = clipboard
            clipboard = pyperclip.paste()

            # Run checkout from clipboard when clipboard changes or first execution
            if clipboard != old or not started:
                if not started:
                    started = True
                clipboard_try()

            time.sleep(0.1)
    except KeyboardInterrupt:
        # Stop confirmation message
        print('Closing watch mode...')
        return


def main():
    easy_helper.is_logged_in()
    print('Starting watch mode.')

    # Auto-checks changes in the clipboard and interrupt if KeyboardInterrupt
    watcher()



# coding: utf-8

from __future__ import print_function, unicode_literals

import tst_wrapper
import easy_helper

import os
import json
import sys


def match_exercise(directory):
    exercises = tst_wrapper.get_exercises()
    os.chdir(directory + '/.tst')
    iid = ''
    result = {}
    with open('tst.json', 'r') as f:
        tst_stats = json.load(f)
        # tst.json stores iid as an integer
        # but tst api returns a "key" as a string,
        # hence we convert to str early
        iid = str(tst_stats['iid'])

    for e in exercises:
        if e['key'] == iid:
            result = e

    return result


def organize(directory, ask=True):
    exercise = match_exercise(directory)
    if exercise == {}:
        return

    destination = easy_helper.exercise_path(exercise)

    if directory == destination:
        print('{} <- Already organized...'.format(directory))
        return

    print('{} -> {}'.format(directory, destination))

    if ask:
        answer = raw_input('Accept: ')

        if answer.lower() != 'y':
            sys.stdout.write("\033[F")  # Cursor up one line
            sys.stdout.write("\033[K")  # Clear to the end of line
            sys.stdout.write("\033[F")  # Cursor up one line
            sys.stdout.write("\033[K")  # Clear to the end of line
            return

        # Clear y answer
        sys.stdout.write("\033[F")  # Cursor up one line
        sys.stdout.write("\033[K")  # Clear to the end of line

    easy_helper.rename_directory(directory, destination)


def get_tst_directories(path):
    result = []
    loading = ['-', '\\', '|', '/']
    current = 0
    for d, _, _ in os.walk(os.path.expanduser(path)):
        current_icon = loading[current % len(loading)]
        print('[{}]'.format(current_icon))
        sys.stdout.write("\033[F")  # Cursor up one line
        sys.stdout.write("\033[K")  # Clear to the end of line
        if '.tst' in os.listdir(d):
            # Skip path if its home, since ~/.tst is tst's config dir
            if d == os.path.expanduser('~/'):
                continue
            result.append(d)
        current += 1

    return result


def main(ask, base_path):
    """Check for tst directories in OS"""
    print('Looking for tst directories under {}'.format(os.path.expanduser(base_path)))
    directories = get_tst_directories(base_path)

    if ask:
        print('Press Y if you agree with the changes. Any other key to skip that folder.')

    for d in directories:
        organize(d, ask)


if __name__ == '__main__':
    main(True, '~/')



# coding: utf-8

from __future__ import print_function, unicode_literals

import os
import subprocess
import json
import requests

import easy_helper


CACHE = {}
EXERCISES_DIR = 'exercicios_tst'


def checkout(exercise):
    code = exercise['checkout_name']
    # Define the tst_root path in the function
    path = os.path.join(os.path.expanduser('~/'),
                        EXERCISES_DIR,
                        easy_helper.format_unit(exercise))

    # Create directory if it's not created already
    if not os.path.isdir(path):
        os.makedirs(path)

    # Change directory to the path
    os.chdir(path)

    # Try to checkout
    tst_out = subprocess.check_output([
        'tst',
        'checkout',
        code
    ])

    # Check checkout vality
    if 'token signature.' in tst_out:
        raise RuntimeError('Not logged in tst.')
    if 'No object' in tst_out:
        raise ValueError('Invalid checkout code.')

    return os.path.join(path, code)


def get_exercises():
    # Return all exercises
    global CACHE
    result = CACHE or update_cache()
    return result['assignments']


def get_exercise_stats(code):
    exercises = get_exercises()

    # Try to get from current cached exercises first
    exercise = [e for e in exercises if e['checkout_name'] == code]

    if len(exercise) == 1:
        return exercise[0]

    # If its not on cache, update current assignments
    exercises = update_cache()['assignments']

    exercise = [e for e in exercises if e['checkout_name'] == code][0]

    return exercise


def auth_key():
    # Define path to the tst config.json file
    tst_path = os.path.expanduser('~/.tst/')
    os.chdir(tst_path)
    # Reads config.json file
    with open('config.json', 'r') as f:
        config = json.load(f)

    # Return access code
    return config['access_token']


def request_to_tst():
    # Requirements definition
    auth = auth_key()
    auth_header = {'Authorization': 'Bearer {}'.format(auth)}
    url = 'http://backend.tst-online.appspot.com/api'

    # Request itself
    response = requests.get(url, headers=auth_header)

    return response


def update_cache():
    response = request_to_tst()

    global CACHE

    if response.status_code == 400:
        raise RuntimeError('Not logged in tst.')

    CACHE = response.json()

    return CACHE


def main(code):
    global CACHE

    exercise = get_exercise_stats(code)
    if len(exercise) == 0:
        raise IndexError
    code, label, name = exercise['checkout_name'], exercise['label'], exercise['name']

    print('Checking out "{}": "{}"'.format(code, label))
    original_path = checkout(exercise)
    destination_path = easy_helper.exercise_path(exercise)
    final_path = easy_helper.rename_directory(original_path, destination_path)

    student = {'name': CACHE['name'], 'email': CACHE['email']}

    easy_helper.create_exercise_file(name, final_path, exercise, student)

    print('''Successful. Path is:
{}
'''.format(final_path))



#!/usr/bin/env python

from setuptools import setup

execfile('easy_tst/__version__.py')

setup(name='easy_tst',
      packages=['easy_tst'],
      version=__version__,
      description='tst automatically',
      author='Bruno Siqueira, Joao Espindula',
      author_email='bruno.siqueira@ccc.ufcg.edu.br',
      url='https://github.com/brobsc/easy_tst',
      download_url='https://github.com/brobsc/easy_tst/archive/{}.tar.gz'.format(__version__),
      install_requires=[
          'pyperclip',
          'requests',
      ],
      scripts=['bin/easy_tst'],
      )
